@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: collect.py 1136 2007-02-23 05:25:10Z pronovic $
+# Revision : $Id: collect.py 741 2007-03-25 16:18:22Z pronovic $
 # Purpose  : Implements the standard 'collect' action.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -95,7 +95,7 @@ def executeCollect(configPath, options, config):
    @raise ValueError: Under many generic error conditions
    @raise TarError: If there is a problem creating a tar file
    """
-   logger.debug("Executing collect action.")
+   logger.debug("Executing the 'collect' action.")
    if config.options is None or config.collect is None:
       raise ValueError("Collect configuration is not properly filled in.")
    if ((config.collect.collectFiles is None or len(config.collect.collectFiles) < 1) and
