@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Official Cedar Backup Extensions
-# Revision : $Id: split.py 1139 2007-02-24 04:16:03Z pronovic $
+# Revision : $Id: split.py 770 2007-09-20 03:07:07Z pronovic $
 # Purpose  : Provides an extension to split up large files in staging directories.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -669,6 +669,8 @@ def _splitFile(sourcePath, splitSize, backupUser, backupGroup, removeSource=Fals
          raise IOError("Error [%d] calling split for [%s]." % (result, sourcePath))
       pattern = re.compile(r"(creating file `)(%s)(.*)(')" % prefix)
       match = pattern.search(output[-1:][0])
+      if match is None:
+         raise IOError("Unable to parse output from split command.")
       value = int(match.group(3).strip())
       for index in range(0, value):
          path = "%s%05d" % (prefix, index)
