@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: filesystem.py 1030 2007-01-03 04:59:12Z pronovic $
+# Revision : $Id: filesystem.py 761 2007-07-05 03:04:54Z pronovic $
 # Purpose  : Provides filesystem-related objects.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -57,7 +57,8 @@ import tarfile
 
 # Cedar Backup modules
 from CedarBackup2.knapsack import firstFit, bestFit, worstFit, alternateFit
-from CedarBackup2.util import AbsolutePathList, ObjectTypeList, UnorderedList, calculateFileAge, encodePath
+from CedarBackup2.util import AbsolutePathList, ObjectTypeList, UnorderedList, RegexList
+from CedarBackup2.util import removeKeys, displayBytes, calculateFileAge, encodePath
 
 
 ########################################################################
@@ -137,8 +138,8 @@ class FilesystemList(list):
       self.excludeLinks = False
       self.excludeDirs = False
       self.excludePaths = []
-      self.excludePatterns = []
-      self.excludeBasenamePatterns = []
+      self.excludePatterns = RegexList()
+      self.excludeBasenamePatterns = RegexList()
       self.ignoreFile = None
 
 
@@ -216,7 +217,7 @@ class FilesystemList(list):
       Property target used to set the exclude patterns list.
       A C{None} value is converted to an empty list.
       """
-      self._excludePatterns = []
+      self._excludePatterns = RegexList()
       if value is not None:
          self._excludePatterns.extend(value)
 
@@ -231,7 +232,7 @@ class FilesystemList(list):
       Property target used to set the exclude basename patterns list.
       A C{None} value is converted to an empty list.
       """
-      self._excludeBasenamePatterns = []
+      self._excludeBasenamePatterns = RegexList()
       if value is not None:
          self._excludeBasenamePatterns.extend(value)
 
@@ -302,10 +303,10 @@ class FilesystemList(list):
          logger.debug("Path [%s] is excluded based on excludePaths." % path)
          return 0
       for pattern in self.excludePatterns:
-         if re.compile(r"^%s$" % pattern).match(path):
+         if re.compile(r"^%s$" % pattern).match(path): # safe to assume all are valid due to RegexList
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return 0
-      for pattern in self.excludeBasenamePatterns:
+      for pattern in self.excludeBasenamePatterns: # safe to assume all are valid due to RegexList
          if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
             logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
             return 0
@@ -344,11 +345,11 @@ class FilesystemList(list):
       if path in self.excludePaths:
          logger.debug("Path [%s] is excluded based on excludePaths." % path)
          return 0
-      for pattern in self.excludePatterns:
+      for pattern in self.excludePatterns: # safe to assume all are valid due to RegexList
          if re.compile(r"^%s$" % pattern).match(path):
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return 0
-      for pattern in self.excludeBasenamePatterns:
+      for pattern in self.excludeBasenamePatterns: # safe to assume all are valid due to RegexList
          if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
             logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
             return 0
@@ -425,11 +426,11 @@ class FilesystemList(list):
       if path in self.excludePaths:
          logger.debug("Path [%s] is excluded based on excludePaths." % path)
          return added
-      for pattern in self.excludePatterns:
+      for pattern in self.excludePatterns: # safe to assume all are valid due to RegexList
          if re.compile(r"^%s$" % pattern).match(path):
             logger.debug("Path [%s] is excluded based on pattern [%s]." % (path, pattern))
             return added
-      for pattern in self.excludeBasenamePatterns:
+      for pattern in self.excludeBasenamePatterns: # safe to assume all are valid due to RegexList
          if re.compile(r"^%s$" % pattern).match(os.path.basename(path)):
             logger.debug("Path [%s] is excluded based on basename pattern [%s]." % (path, pattern))
             return added
@@ -474,6 +475,7 @@ class FilesystemList(list):
       @param pattern: Regular expression pattern representing entries to remove
 
       @return: Number of entries removed
+      @raise ValueError: If the passed-in pattern is not a valid regular expression.
       """
       removed = 0
       if pattern is None:
@@ -483,7 +485,10 @@ class FilesystemList(list):
                logger.debug("Removed path [%s] from list." % entry)
                removed += 1
       else:
-         compiled = re.compile(pattern)
+         try:
+            compiled = re.compile(pattern)
+         except re.error:
+            raise ValueError("Pattern is not a valid regular expression.")
          for entry in self[:]:
             if os.path.exists(entry) and os.path.isfile(entry):
                if compiled.match(entry):
@@ -512,6 +517,7 @@ class FilesystemList(list):
       @param pattern: Regular expression pattern representing entries to remove
 
       @return: Number of entries removed
+      @raise ValueError: If the passed-in pattern is not a valid regular expression.
       """
       removed = 0
       if pattern is None:
@@ -521,7 +527,10 @@ class FilesystemList(list):
                logger.debug("Removed path [%s] from list." % entry)
                removed += 1
       else:
-         compiled = re.compile(pattern)
+         try:
+            compiled = re.compile(pattern)
+         except re.error:
+            raise ValueError("Pattern is not a valid regular expression.")
          for entry in self[:]:
             if os.path.exists(entry) and os.path.isdir(entry):
                if compiled.match(entry):
@@ -548,6 +557,7 @@ class FilesystemList(list):
       @param pattern: Regular expression pattern representing entries to remove
 
       @return: Number of entries removed
+      @raise ValueError: If the passed-in pattern is not a valid regular expression.
       """
       removed = 0
       if pattern is None:
@@ -557,7 +567,10 @@ class FilesystemList(list):
                logger.debug("Removed path [%s] from list." % entry)
                removed += 1
       else:
-         compiled = re.compile(pattern)
+         try:
+            compiled = re.compile(pattern)
+         except re.error:
+            raise ValueError("Pattern is not a valid regular expression.")
          for entry in self[:]:
             if os.path.exists(entry) and os.path.islink(entry):
                if compiled.match(entry):
@@ -586,9 +599,13 @@ class FilesystemList(list):
       @param pattern: Regular expression pattern representing entries to remove
 
       @return: Number of entries removed.
+      @raise ValueError: If the passed-in pattern is not a valid regular expression.
       """
+      try:
+         compiled = re.compile(pattern)
+      except re.error:
+         raise ValueError("Pattern is not a valid regular expression.")
       removed = 0
-      compiled = re.compile(pattern)
       for entry in self[:]:
          if compiled.match(entry):
             self.remove(entry)
@@ -644,6 +661,27 @@ class FilesystemList(list):
       return True
 
 
+########################################################################
+# SpanItem class definition
+########################################################################
+
+class SpanItem(object):
+   """
+   Item returned by L{BackupFileList.generateSpan}.
+   """
+   def __init__(self, fileList, size, capacity, utilization):
+      """
+      Create object.
+      @param fileList: List of files
+      @param size: Size (in bytes) of files
+      @param utilization: Utilization, as a percentage (0-100)
+      """
+      self.fileList = fileList
+      self.size = size
+      self.capacity = capacity
+      self.utilization = utilization
+
+
 ########################################################################
 # BackupFileList class definition
 ########################################################################
@@ -849,17 +887,89 @@ class BackupFileList(FilesystemList):
       @return: Copy of list with total size no larger than indicated capacity
       @raise ValueError: If the algorithm is invalid.
       """
+      table = self._getKnapsackTable()
+      function = BackupFileList._getKnapsackFunction(algorithm)
+      return function(table, capacity)[0]
+
+   def generateSpan(self, capacity, algorithm="worst_fit"):
+      """
+      Splits the list of items into sub-lists that fit in a given capacity.
+
+      Sometimes, callers need split to a backup file list into a set of smaller
+      lists.  For instance, you could use this to "span" the files across a set
+      of discs.
+
+      The fitting is done using the functions in the knapsack module.  By
+      default, the first fit algorithm is used, but you can also choose
+      from best fit, worst fit and alternate fit.
+
+      @note: If any of your items are larger than the capacity, then it won't
+      be possible to find a solution.  In this case, a value error will be
+      raised.
+
+      @param capacity: Maximum capacity among the files in the new list
+      @type capacity: Integer, in bytes
+
+      @param algorithm: Knapsack (fit) algorithm to use
+      @type algorithm: One of "first_fit", "best_fit", "worst_fit", "alternate_fit"
+
+      @return: List of L{SpanItem} objects.
+
+      @raise ValueError: If the algorithm is invalid.
+      @raise ValueError: If it's not possible to fit some items
+      """
+      spanItems = []
+      function = BackupFileList._getKnapsackFunction(algorithm)
+      table = self._getKnapsackTable(capacity)
+      iteration = 0
+      while len(table) > 0:
+         iteration += 1
+         fit = function(table, capacity)
+         if len(fit[0]) == 0:
+            # Should never happen due to validations in _convertToKnapsackForm(), but let's be safe
+            raise ValueError("After iteration %d, unable to add any new items." % iteration)
+         removeKeys(table, fit[0])
+         utilization = (float(fit[1])/float(capacity))*100.0
+         item = SpanItem(fit[0], fit[1], capacity, utilization)
+         spanItems.append(item)
+      return spanItems
+
+   def _getKnapsackTable(self, capacity=None):
+      """
+      Converts the list into the form needed by the knapsack algorithms.
+      @return: Dictionary mapping file name to tuple of (file path, file size).
+      """
       table = { }
       for entry in self:
          if os.path.islink(entry):
             table[entry] = (entry, 0.0)
          elif os.path.isfile(entry):
-            table[entry] = (entry, float(os.stat(entry).st_size))
-      if algorithm == "first_fit": return firstFit(table, capacity)[0]
-      elif algorithm == "best_fit": return bestFit(table, capacity)[0]
-      elif algorithm == "worst_fit": return worstFit(table, capacity)[0]
-      elif algorithm == "alternate_fit": return alternateFit(table, capacity)[0]
-      else: raise ValueError("Algorithm [%s] is invalid." % algorithm);
+            size = float(os.stat(entry).st_size)
+            if capacity is not None:
+               if size > capacity:
+                  raise ValueError("File [%s] cannot fit in capacity %s." % (entry, displayBytes(capacity)))
+            table[entry] = (entry, size)
+      return table
+
+   def _getKnapsackFunction(algorithm):
+      """
+      Returns a reference to the function associated with an algorithm name.
+      Algorithm name must be one of "first_fit", "best_fit", "worst_fit", "alternate_fit"
+      @param algorithm: Name of the algorithm
+      @return: Reference to knapsack function
+      @raise ValueError: If the algorithm name is unknown.
+      """
+      if algorithm == "first_fit":
+         return firstFit
+      elif algorithm == "best_fit":
+         return bestFit
+      elif algorithm == "worst_fit":
+         return worstFit
+      elif algorithm == "alternate_fit":
+         return alternateFit
+      else:
+         raise ValueError("Algorithm [%s] is invalid." % algorithm);
+   _getKnapsackFunction = staticmethod(_getKnapsackFunction)
 
    def generateTarfile(self, path, mode='tar', ignore=False, flat=False):
       """
@@ -1266,7 +1376,7 @@ def compareContents(path1, path2, verbose=False):
    @type path1: String representing a path on disk
 
    @param path2: First path to compare.
-   @type path12 String representing a path on disk
+   @type path2: String representing a path on disk
 
    @param verbose: Indicates whether a verbose response should be given.
    @type verbose: Boolean
@@ -1282,18 +1392,35 @@ def compareContents(path1, path2, verbose=False):
       path2List = BackupFileList()
       path2List.addDirContents(path2)
       path2Digest = path2List.generateDigestMap(stripPrefix=normalizeDir(path2))
-      if not verbose:
-         if path1Digest != path2Digest:
-            raise ValueError("Consistency check failed.")
-      else:
-         list1 = UnorderedList(path1Digest.keys())
-         list2 = UnorderedList(path2Digest.keys())
-         if list1 != list2:
-            raise ValueError("Directories contain a different set of files.")
-         for key in list1:
-            if path1Digest[key] != path2Digest[key]:
-               raise ValueError("File contents for [%s] vary between directories." % key)
+      compareDigestMaps(path1Digest, path2Digest, verbose)
    except IOError, e:
       logger.error("I/O error encountered during consistency check.")
       raise e
 
+def compareDigestMaps(digest1, digest2, verbose=False):
+   """
+   Compares two digest maps and throws an exception if they differ.
+
+   @param digest1: First digest to compare.
+   @type digest1: Digest as returned from BackupFileList.generateDigestMap()
+
+   @param digest2: Second digest to compare.
+   @type digest2: Digest as returned from BackupFileList.generateDigestMap()
+
+   @param verbose: Indicates whether a verbose response should be given.
+   @type verbose: Boolean
+
+   @raise ValueError: If the two directories are not equivalent.
+   """
+   if not verbose:
+      if digest1 != digest2:
+         raise ValueError("Consistency check failed.")
+   else:
+      list1 = UnorderedList(digest1.keys())
+      list2 = UnorderedList(digest2.keys())
+      if list1 != list2:
+         raise ValueError("Directories contain a different set of files.")
+      for key in list1:
+         if digest1[key] != digest2[key]:
+            raise ValueError("File contents for [%s] vary between directories." % key)
+
